"""
Procedural Street-Builder
#Draw a pathspline and drag it as child of the Street-Builder
by Thomas Bösl
Am Stadtpark 22
09120 Chemnitz
www.3Deoskill.com
mail:info@3Deoskill.com
"""

import c4d
from c4d import gui
import os


# Main function
def main():
    if os.path.exists(os.path.join(FOLDER_PATH, "Street-Builder.tif")):
        path = os.path.join(FOLDER_PATH, "Street-Builder.tif")
    else:
        path = "5186"

    python = c4d.BaseObject(1023866)
    python[c4d.ID_BASELIST_NAME] = "STREET-BUILDER"
    python[c4d.ID_BASELIST_ICON_FILE] = path
    python[c4d.OPYTHON_OPTIMIZE] = False

    # Make UserData Interface
    first_group = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    first_group[c4d.DESC_NAME] = "ST-BUILDER"
    FirstGroupId = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                              c4d.DescLevel(1, c4d.DTYPE_GROUP, 0))
    first_group[c4d.DESC_TITLEBAR] = 1
    first_group[c4d.DESC_GUIOPEN] = 1

    first_group[c4d.DESC_PARENTGROUP] = c4d.DescID(0)

    # Make UserData Interface
    Group1 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    Group1[c4d.DESC_NAME] = "Street Parameters"
    Group1Id = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                          c4d.DescLevel(2, c4d.DTYPE_GROUP, 0))
    Group1[c4d.DESC_TITLEBAR] = 1
    Group1[c4d.DESC_GUIOPEN] = 1

    Group1[c4d.DESC_PARENTGROUP] = FirstGroupId

    pave_width = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    pave_width[c4d.DESC_NAME] = "Pavement Width"
    pave_width[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    pave_width[c4d.DESC_STEP] = 1
    pave_width[c4d.DESC_MIN] = 0
    pave_width[c4d.DESC_PARENTGROUP] = Group1Id

    curb_width = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    curb_width[c4d.DESC_NAME] = "Curbstone Width"
    curb_width[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    curb_width[c4d.DESC_STEP] = 1
    curb_width[c4d.DESC_MIN] = 0
    curb_width[c4d.DESC_PARENTGROUP] = Group1Id

    street_width = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    street_width[c4d.DESC_NAME] = "Street Width"
    street_width[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    street_width[c4d.DESC_STEP] = 1
    street_width[c4d.DESC_MIN] = 0
    street_width[c4d.DESC_PARENTGROUP] = Group1Id

    pave_height = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    pave_height[c4d.DESC_NAME] = "Pavement Height"
    pave_height[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    pave_height[c4d.DESC_STEP] = 1
    pave_height[c4d.DESC_MIN] = 0
    pave_height[c4d.DESC_PARENTGROUP] = Group1Id

    # Group2 Materials
    Group2 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    Group2[c4d.DESC_NAME] = "Street Materials"
    Group2Id = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                          c4d.DescLevel(11, c4d.DTYPE_GROUP, 0))
    Group2[c4d.DESC_TITLEBAR] = 1
    Group2[c4d.DESC_COLUMNS] = 2
    Group2[c4d.DESC_GUIOPEN] = 1
    Group2[c4d.DESC_PARENTGROUP] = FirstGroupId

    pave_mat = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    pave_mat[c4d.DESC_NAME] = "Pavement-Material"
    pave_mat[c4d.DESC_PARENTGROUP] = Group2Id

    curb_mat = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    curb_mat[c4d.DESC_NAME] = "Curbstone-Material"
    curb_mat[c4d.DESC_PARENTGROUP] = Group2Id

    street_mat = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    street_mat[c4d.DESC_NAME] = "Street-Material"
    street_mat[c4d.DESC_PARENTGROUP] = Group2Id

    pave_tile = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR4D)
    pave_tile[c4d.DESC_NAME] = "Texture Tiling"
    pave_tile[c4d.DESC_UNIT] = c4d.DESC_UNIT_REAL
    pave_tile[c4d.DESC_PARENTGROUP] = Group2Id

    curb_tile = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR4D)
    curb_tile[c4d.DESC_NAME] = "Texture Tiling"
    curb_tile[c4d.DESC_UNIT] = c4d.DESC_UNIT_REAL
    curb_tile[c4d.DESC_PARENTGROUP] = Group2Id

    street_tile = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR4D)
    street_tile[c4d.DESC_NAME] = "Texture Tiling"
    street_tile[c4d.DESC_UNIT] = c4d.DESC_UNIT_REAL
    street_tile[c4d.DESC_PARENTGROUP] = Group2Id

    rot = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    rot[c4d.DESC_NAME] = "Fix Rotation"
    rot[c4d.DESC_UNIT] = c4d.DESC_UNIT_DEGREE
    rot[c4d.DESC_STEP] = c4d.utils.DegToRad(1)
    rot[c4d.DESC_PARENTGROUP] = Group1Id

    # Bevel Funktion
    bevel = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    bevel[c4d.DESC_NAME] = "Bevel On/Off"
    bevel[c4d.DESC_PARENTGROUP] = Group1Id

    offset = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    offset[c4d.DESC_NAME] = "Offset"
    offset[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    offset[c4d.DESC_STEP] = 1
    offset[c4d.DESC_MIN] = 0
    offset[c4d.DESC_PARENTGROUP] = Group1Id

    subd = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    subd[c4d.DESC_NAME] = "Subdivision"
    subd[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    subd[c4d.DESC_STEP] = 1
    subd[c4d.DESC_MIN] = 0
    subd[c4d.DESC_PARENTGROUP] = Group1Id

    # object group lamps

    group3 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    group3[c4d.DESC_NAME] = "Street-Lamps"
    Group3Id = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                          c4d.DescLevel(18, c4d.DTYPE_GROUP, 0))
    group3[c4d.DESC_TITLEBAR] = 1
    group3[c4d.DESC_COLUMNS] = 1
    group3[c4d.DESC_GUIOPEN] = 1
    group3[c4d.DESC_PARENTGROUP] = FirstGroupId

    object1 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    object1[c4d.DESC_NAME] = "Streetlamp 1"
    object1[c4d.DESC_PARENTGROUP] = Group3Id

    obj1_goal = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    obj1_goal[c4d.DESC_NAME] = "Look Up"
    obj1_goal[c4d.DESC_PARENTGROUP] = FirstGroupId

    obj1_pitch = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    obj1_pitch[c4d.DESC_NAME] = "Pitch"
    obj1_pitch[c4d.DESC_PARENTGROUP] = FirstGroupId

    obj1_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    obj1_pos[c4d.DESC_NAME] = "Position"
    obj1_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    obj1_pos[c4d.DESC_PARENTGROUP] = Group3Id

    obj1_rot = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    obj1_rot[c4d.DESC_NAME] = "Rotation"
    obj1_rot[c4d.DESC_UNIT] = c4d.DESC_UNIT_DEGREE
    obj1_rot[c4d.DESC_STEP] = c4d.Vector(c4d.utils.DegToRad(1))
    obj1_rot[c4d.DESC_PARENTGROUP] = Group3Id

    obj1_count = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    obj1_count[c4d.DESC_NAME] = "Count"
    obj1_count[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    obj1_count[c4d.DESC_PARENTGROUP] = Group3Id
    obj1_count[c4d.DESC_STEP] = 1
    obj1_count[c4d.DESC_MIN] = 0

    object2 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    object2[c4d.DESC_NAME] = "Streetlamp 2"
    object2[c4d.DESC_PARENTGROUP] = Group3Id

    obj2_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    obj2_pos[c4d.DESC_NAME] = "Position"
    obj2_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    obj2_pos[c4d.DESC_PARENTGROUP] = Group3Id

    obj2_rot = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    obj2_rot[c4d.DESC_NAME] = "Rotation"
    obj2_rot[c4d.DESC_UNIT] = c4d.DESC_UNIT_DEGREE
    obj2_rot[c4d.DESC_STEP] = c4d.Vector(c4d.utils.DegToRad(1))
    obj2_rot[c4d.DESC_PARENTGROUP] = Group3Id

    obj2_count = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    obj2_count[c4d.DESC_NAME] = "Count"
    obj2_count[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    obj2_count[c4d.DESC_PARENTGROUP] = Group3Id
    obj2_count[c4d.DESC_STEP] = 1
    obj2_count[c4d.DESC_MIN] = 0

    # rail===========
    rail_group = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    rail_group[c4d.DESC_NAME] = "RAILS"
    RailGroupId = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                             c4d.DescLevel(27, c4d.DTYPE_GROUP, 0))
    rail_group[c4d.DESC_TITLEBAR] = 1
    rail_group[c4d.DESC_COLUMNS] = 1
    rail_group[c4d.DESC_GUIOPEN] = 1
    rail_group[c4d.DESC_PARENTGROUP] = c4d.DescID(0)

    # Rail1-Data=========================================
    group4 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    group4[c4d.DESC_NAME] = "Rail_1"
    Group4Id = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                          c4d.DescLevel(28, c4d.DTYPE_GROUP, 0))
    group4[c4d.DESC_TITLEBAR] = 1
    group4[c4d.DESC_COLUMNS] = 1
    group4[c4d.DESC_GUIOPEN] = 1
    group4[c4d.DESC_PARENTGROUP] = RailGroupId

    rail = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    rail[c4d.DESC_NAME] = "Rail-Profile"
    rail[c4d.DESC_PARENTGROUP] = Group4Id

    rail_bool = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    rail_bool[c4d.DESC_NAME] = "Rail On/Off"
    rail_bool[c4d.DESC_PARENTGROUP] = Group4Id

    rail1_quick = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    rail1_quick[c4d.DESC_NAME] = "Type"
    rail1_quick[c4d.DESC_CUSTOMGUI] = 200000281  # Quicktab Radio access
    cycle1 = c4d.BaseContainer()
    cycle1[0] = "Type_1"
    cycle1[1] = "Type_2"
    cycle1[2] = "Type_3"
    cycle1[3] = "Type_4"
    cycle1[4] = "Type_5"
    cycle1[5] = "Type_6"
    rail1_quick[c4d.DESC_CYCLE] = cycle1
    rail1_quick[c4d.DESC_PARENTGROUP] = Group4Id

    rail_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    rail_pos[c4d.DESC_NAME] = "Rail Position"
    rail_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    rail_pos[c4d.DESC_PARENTGROUP] = Group4Id

    # rail_rot=c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    # rail_rot[c4d.DESC_NAME]="Rail Rotation"
    # rail_rot[c4d.DESC_UNIT]=c4d.DESC_UNIT_DEGREE
    # rail_rot[c4d.DESC_PARENTGROUP]=Group4Id
    # rail_rot[c4d.DESC_STEP]=c4d.Vector(c4d.utils.DegToRad(1))

    rail_size = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    rail_size[c4d.DESC_NAME] = "Size"
    rail_size[c4d.DESC_UNIT] = c4d.DESC_UNIT_PERCENT
    rail_size[c4d.DESC_STEP] = 0.01
    rail_size[c4d.DESC_PARENTGROUP] = Group4Id

    rail_mirror = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    rail_mirror[c4d.DESC_NAME] = "Mirror"
    rail_mirror[c4d.DESC_PARENTGROUP] = Group4Id

    rail_mat = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    rail_mat[c4d.DESC_NAME] = "Rail Material"
    rail_mat[c4d.DESC_PARENTGROUP] = Group4Id

    rail_tile = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR4D)
    rail_tile[c4d.DESC_NAME] = "Texture Tiling"
    rail_tile[c4d.DESC_UNIT] = c4d.DESC_UNIT_REAL
    rail_tile[c4d.DESC_PARENTGROUP] = Group4Id

    column = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    column[c4d.DESC_NAME] = "Rail Column"
    column[c4d.DESC_PARENTGROUP] = Group4Id

    column_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    column_pos[c4d.DESC_NAME] = "Column Position"
    column_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    column_pos[c4d.DESC_PARENTGROUP] = Group4Id

    column_rot = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    column_rot[c4d.DESC_NAME] = "Column Rotation"
    column_rot[c4d.DESC_UNIT] = c4d.DESC_UNIT_DEGREE
    column_rot[c4d.DESC_STEP] = c4d.Vector(c4d.utils.DegToRad(1))
    column_rot[c4d.DESC_PARENTGROUP] = Group4Id

    column_count = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    column_count[c4d.DESC_NAME] = "Column Count"
    column_count[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    column_count[c4d.DESC_PARENTGROUP] = Group4Id
    column_count[c4d.DESC_STEP] = 1
    column_count[c4d.DESC_MIN] = 0

    # ==============================

    # Rail2-Data=========================================
    group5 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    group5[c4d.DESC_NAME] = "Rail_2"
    Group5Id = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                          c4d.DescLevel(41, c4d.DTYPE_GROUP, 0))
    group5[c4d.DESC_TITLEBAR] = 1
    group5[c4d.DESC_COLUMNS] = 1
    group5[c4d.DESC_GUIOPEN] = 1
    group5[c4d.DESC_PARENTGROUP] = RailGroupId

    rail2 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    rail2[c4d.DESC_NAME] = "Rail-Profile"
    rail2[c4d.DESC_PARENTGROUP] = Group5Id

    rail2_bool = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    rail2_bool[c4d.DESC_NAME] = "Rail On/Off"
    rail2_bool[c4d.DESC_PARENTGROUP] = Group5Id

    rail2_quick = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    rail2_quick[c4d.DESC_NAME] = "Type"
    rail2_quick[c4d.DESC_CUSTOMGUI] = 200000281  # Quicktab Radio access
    cycle2 = c4d.BaseContainer()
    cycle2[0] = "Type_1"
    cycle2[1] = "Type_2"
    cycle2[2] = "Type_3"
    cycle2[3] = "Type_4"
    cycle2[4] = "Type_5"
    cycle2[5] = "Type_6"
    rail2_quick[c4d.DESC_CYCLE] = cycle2
    rail2_quick[c4d.DESC_PARENTGROUP] = Group5Id

    rail2_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    rail2_pos[c4d.DESC_NAME] = "Rail Position"
    rail2_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    rail2_pos[c4d.DESC_PARENTGROUP] = Group5Id

    # rail_rot=c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    # rail_rot[c4d.DESC_NAME]="Rail Rotation"
    # rail_rot[c4d.DESC_UNIT]=c4d.DESC_UNIT_DEGREE
    # rail_rot[c4d.DESC_PARENTGROUP]=Group4Id
    # rail_rot[c4d.DESC_STEP]=c4d.Vector(c4d.utils.DegToRad(1))

    rail2_size = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    rail2_size[c4d.DESC_NAME] = "Size"
    rail2_size[c4d.DESC_UNIT] = c4d.DESC_UNIT_PERCENT
    rail2_size[c4d.DESC_STEP] = 0.01
    rail2_size[c4d.DESC_PARENTGROUP] = Group5Id

    rail2_mirror = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    rail2_mirror[c4d.DESC_NAME] = "Mirror"
    rail2_mirror[c4d.DESC_PARENTGROUP] = Group5Id

    rail2_mat = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    rail2_mat[c4d.DESC_NAME] = "Rail Material"
    rail2_mat[c4d.DESC_PARENTGROUP] = Group5Id

    rail2_tile = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR4D)
    rail2_tile[c4d.DESC_NAME] = "Texture Tiling"
    rail2_tile[c4d.DESC_UNIT] = c4d.DESC_UNIT_REAL
    rail2_tile[c4d.DESC_PARENTGROUP] = Group5Id

    column2 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    column2[c4d.DESC_NAME] = "Rail Column"
    column2[c4d.DESC_PARENTGROUP] = Group5Id

    column2_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    column2_pos[c4d.DESC_NAME] = "Column Position"
    column2_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    column2_pos[c4d.DESC_PARENTGROUP] = Group5Id

    column2_rot = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    column2_rot[c4d.DESC_NAME] = "Column Rotation"
    column2_rot[c4d.DESC_UNIT] = c4d.DESC_UNIT_DEGREE
    column2_rot[c4d.DESC_STEP] = c4d.Vector(c4d.utils.DegToRad(1))
    column2_rot[c4d.DESC_PARENTGROUP] = Group5Id

    column2_count = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    column2_count[c4d.DESC_NAME] = "Column Count"
    column2_count[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    column2_count[c4d.DESC_PARENTGROUP] = Group5Id
    column2_count[c4d.DESC_STEP] = 1
    column2_count[c4d.DESC_MIN] = 0

    # ==============================

    # Rail2-Data=========================================
    group6 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_GROUP)
    group6[c4d.DESC_NAME] = "Rail_3"
    Group6Id = c4d.DescID(c4d.DescLevel(c4d.ID_USERDATA, c4d.DTYPE_SUBCONTAINER, 0),
                          c4d.DescLevel(54, c4d.DTYPE_GROUP, 0))
    group6[c4d.DESC_TITLEBAR] = 1
    group6[c4d.DESC_COLUMNS] = 1
    group6[c4d.DESC_GUIOPEN] = 1
    group6[c4d.DESC_PARENTGROUP] = RailGroupId

    rail3 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    rail3[c4d.DESC_NAME] = "Rail-Profile"
    rail3[c4d.DESC_PARENTGROUP] = Group6Id

    rail3_bool = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    rail3_bool[c4d.DESC_NAME] = "Rail On/Off"
    rail3_bool[c4d.DESC_PARENTGROUP] = Group6Id

    rail3_quick = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    rail3_quick[c4d.DESC_NAME] = "Type"
    rail3_quick[c4d.DESC_CUSTOMGUI] = 200000281  # Quicktab Radio access
    cycle3 = c4d.BaseContainer()
    cycle3[0] = "Type_1"
    cycle3[1] = "Type_2"
    cycle3[2] = "Type_3"
    cycle3[3] = "Type_4"
    cycle3[4] = "Type_5"
    cycle3[5] = "Type_6"
    rail3_quick[c4d.DESC_CYCLE] = cycle3
    rail3_quick[c4d.DESC_PARENTGROUP] = Group6Id

    rail3_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    rail3_pos[c4d.DESC_NAME] = "Rail Position"
    rail3_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    rail3_pos[c4d.DESC_PARENTGROUP] = Group6Id

    # rail_rot=c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    # rail_rot[c4d.DESC_NAME]="Rail Rotation"
    # rail_rot[c4d.DESC_UNIT]=c4d.DESC_UNIT_DEGREE
    # rail_rot[c4d.DESC_PARENTGROUP]=Group4Id
    # rail_rot[c4d.DESC_STEP]=c4d.Vector(c4d.utils.DegToRad(1))

    rail3_size = c4d.GetCustomDataTypeDefault(c4d.DTYPE_REAL)
    rail3_size[c4d.DESC_NAME] = "Size"
    rail3_size[c4d.DESC_UNIT] = c4d.DESC_UNIT_PERCENT
    rail3_size[c4d.DESC_STEP] = 0.01
    rail3_size[c4d.DESC_PARENTGROUP] = Group6Id

    rail3_mirror = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BOOL)
    rail3_mirror[c4d.DESC_NAME] = "Mirror"
    rail3_mirror[c4d.DESC_PARENTGROUP] = Group6Id

    rail3_mat = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    rail3_mat[c4d.DESC_NAME] = "Rail Material"
    rail3_mat[c4d.DESC_PARENTGROUP] = Group6Id

    rail3_tile = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR4D)
    rail3_tile[c4d.DESC_NAME] = "Texture Tiling"
    rail3_tile[c4d.DESC_UNIT] = c4d.DESC_UNIT_REAL
    rail3_tile[c4d.DESC_PARENTGROUP] = Group6Id

    column3 = c4d.GetCustomDataTypeDefault(c4d.DTYPE_BASELISTLINK)
    column3[c4d.DESC_NAME] = "Rail Column"
    column3[c4d.DESC_PARENTGROUP] = Group6Id

    column3_pos = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    column3_pos[c4d.DESC_NAME] = "Column Position"
    column3_pos[c4d.DESC_UNIT] = c4d.DESC_UNIT_METER
    column3_pos[c4d.DESC_PARENTGROUP] = Group6Id

    column3_rot = c4d.GetCustomDataTypeDefault(c4d.DTYPE_VECTOR)
    column3_rot[c4d.DESC_NAME] = "Column Rotation"
    column3_rot[c4d.DESC_UNIT] = c4d.DESC_UNIT_DEGREE
    column3_rot[c4d.DESC_STEP] = c4d.Vector(c4d.utils.DegToRad(1))
    column3_rot[c4d.DESC_PARENTGROUP] = Group6Id

    column3_count = c4d.GetCustomDataTypeDefault(c4d.DTYPE_LONG)
    column3_count[c4d.DESC_NAME] = "Column Count"
    column3_count[c4d.DESC_UNIT] = c4d.DESC_UNIT_INT
    column3_count[c4d.DESC_PARENTGROUP] = Group6Id
    column3_count[c4d.DESC_STEP] = 1
    column3_count[c4d.DESC_MIN] = 0

    # ==============================

    # Add Userdata
    FirstGroup = python.AddUserData(first_group)
    group1 = python.AddUserData(Group1)

    Pave_Width = python.AddUserData(pave_width)
    python[Pave_Width] = 80

    Curb_Width = python.AddUserData(curb_width)
    python[Curb_Width] = 20

    Street_Width = python.AddUserData(street_width)
    python[Street_Width] = 400

    Pave_Height = python.AddUserData(pave_height)
    python[Pave_Height] = 20

    # Bevel AddUserdata
    Rot = python.AddUserData(rot)
    python[Rot] = c4d.utils.DegToRad(-180)

    Bevel = python.AddUserData(bevel)  # 14
    python[Bevel] = False

    Offset = python.AddUserData(offset)
    python[Offset] = 1

    Subd = python.AddUserData(subd)
    python[Subd] = 0
    # ==================

    # Materials Userdata
    group2 = python.AddUserData(Group2)

    Pave_Mat = python.AddUserData(pave_mat)

    Pave_Tile = python.AddUserData(pave_tile)
    python[Pave_Tile] = c4d.Vector4d(0, 100, 0, 100)

    Curb_Mat = python.AddUserData(curb_mat)
    Curb_Tile = python.AddUserData(curb_tile)
    python[Curb_Tile] = c4d.Vector4d(0, 100, 0, 100)

    Street_Mat = python.AddUserData(street_mat)
    Street_Tile = python.AddUserData(street_tile)
    python[Street_Tile] = c4d.Vector4d(0, 100, 0, 100)
    # ===========================

    Group3 = python.AddUserData(group3)
    Object1 = python.AddUserData(object1)
    Obj1_pos = python.AddUserData(obj1_pos)
    Obj1_rot = python.AddUserData(obj1_rot)
    Obj1_count = python.AddUserData(obj1_count)
    python[Obj1_count] = 5

    Object2 = python.AddUserData(object2)
    Obj2_pos = python.AddUserData(obj2_pos)
    Obj2_rot = python.AddUserData(obj2_rot)
    Obj2_count = python.AddUserData(obj2_count)
    python[Obj2_count] = 5

    # Rail1=======================

    RailGroup = python.AddUserData(rail_group)
    Group4 = python.AddUserData(group4)

    Rail_Bool = python.AddUserData(rail_bool)
    python[Rail_Bool] = 0
    Rail1_Quick = python.AddUserData(rail1_quick)

    Rail = python.AddUserData(rail)
    Rail_pos = python.AddUserData(rail_pos)
    # Rail_rot=python.AddUserData(rail_rot)

    Rail_Size = python.AddUserData(rail_size)
    python[Rail_Size] = 1
    Rail_Mirror = python.AddUserData(rail_mirror)
    python[Rail_Mirror] = False

    Rail_mat = python.AddUserData(rail_mat)
    Rail_tile = python.AddUserData(rail_tile)
    python[Rail_tile] = c4d.Vector4d(0, 100, 0, 100)

    # Sep1=python.AddUserData(sep1)

    Column = python.AddUserData(column)
    Column_pos = python.AddUserData(column_pos)
    Column_rot = python.AddUserData(column_rot)
    Column_count = python.AddUserData(column_count)
    python[Column_count] = 5

    # =============================================

    # Rail2=======================

    Group5 = python.AddUserData(group5)

    Rail2_Bool = python.AddUserData(rail2_bool)
    python[Rail2_Bool] = 0
    Rail2_Quick = python.AddUserData(rail2_quick)

    Rail2 = python.AddUserData(rail2)
    Rail2_pos = python.AddUserData(rail2_pos)
    # Rail_rot=python.AddUserData(rail_rot)

    Rail2_Size = python.AddUserData(rail2_size)
    python[Rail2_Size] = 1
    Rail2_Mirror = python.AddUserData(rail2_mirror)
    python[Rail2_Mirror] = False

    Rail2_mat = python.AddUserData(rail2_mat)
    Rail2_tile = python.AddUserData(rail2_tile)
    python[Rail2_tile] = c4d.Vector4d(0, 100, 0, 100)

    # Sep1=python.AddUserData(sep1)

    Column2 = python.AddUserData(column2)
    Column2_pos = python.AddUserData(column2_pos)
    Column2_rot = python.AddUserData(column2_rot)
    Column2_count = python.AddUserData(column2_count)
    python[Column2_count] = 5

    # =============================================

    # Rail3=======================

    Group6 = python.AddUserData(group6)

    Rail3_Bool = python.AddUserData(rail3_bool)
    python[Rail3_Bool] = 0
    Rail3_Quick = python.AddUserData(rail3_quick)

    Rail3 = python.AddUserData(rail3)
    Rail3_pos = python.AddUserData(rail3_pos)
    # Rail_rot=python.AddUserData(rail_rot)

    Rail3_Size = python.AddUserData(rail3_size)
    python[Rail3_Size] = 1
    Rail3_Mirror = python.AddUserData(rail3_mirror)
    python[Rail3_Mirror] = False

    Rail3_mat = python.AddUserData(rail3_mat)
    Rail3_tile = python.AddUserData(rail3_tile)
    python[Rail3_tile] = c4d.Vector4d(0, 100, 0, 100)

    # Sep1=python.AddUserData(sep1)

    Column3 = python.AddUserData(column3)
    Column3_pos = python.AddUserData(column3_pos)
    Column3_rot = python.AddUserData(column3_rot)
    Column3_count = python.AddUserData(column3_count)
    python[Column3_count] = 5

    # =============================================

    # ===Pitch Installer
    Obj1_Goal = python.AddUserData(obj1_goal)
    python[Obj1_Goal] = 0

    Obj1_Pitch = python.AddUserData(obj1_pitch)
    python[Obj1_Pitch] = 0

    python[c4d.OPYTHON_CODE] = '''




# ==============================================
"""
Procedural Street-Builder
#Draw a pathspline and drag it as child of the Street-Builder
by Thomas Bösl
Am Stadtpark 22
09120 Chemnitz
www.3Deoskill.com
mail:info@3Deoskill.com
"""
import c4d
import maxon
import os


def Spline():
    null = c4d.BaseObject(c4d.Onull)
    spline = c4d.SplineObject(2, c4d.SPLINETYPE_LINEAR)

    # street
    spline.SetPoint(0, c4d.Vector(-street_width / 2 + pave_width + curb_width, 0.1, 0))
    spline.SetPoint(1, c4d.Vector(street_width / 2 - pave_width - curb_width, 0.1, 0))

    # curb spline
    curb_left_spline = c4d.SplineObject(3, c4d.SPLINETYPE_LINEAR)
    curb_left_spline.SetPoint(0, spline.GetPoint(0))
    curb_left_spline.SetPoint(1, curb_left_spline.GetPoint(0) + c4d.Vector(0, pave_height, 0))
    curb_left_spline.SetPoint(2, curb_left_spline.GetPoint(1) - c4d.Vector(curb_width, 0, 0))

    # pave left spline
    pave_left_spline = c4d.SplineObject(3, c4d.SPLINETYPE_LINEAR)
    pave_left_spline.SetPoint(0, curb_left_spline.GetPoint(2))
    pave_left_spline.SetPoint(1, pave_left_spline.GetPoint(0) - c4d.Vector(pave_width, 0, 0))
    pave_left_spline.SetPoint(2, pave_left_spline.GetPoint(1) - c4d.Vector(0, pave_height + 0.1, 0))

    # curb right spline
    curb_right_spline = c4d.SplineObject(3, c4d.SPLINETYPE_LINEAR)
    curb_right_spline.SetPoint(0, spline.GetPoint(1))
    curb_right_spline.SetPoint(1, curb_right_spline.GetPoint(0) + c4d.Vector(0, pave_height, 0))
    curb_right_spline.SetPoint(2, curb_right_spline.GetPoint(1) + c4d.Vector(curb_width, 0, 0))

    # pave right spline
    pave_right_spline = c4d.SplineObject(3, c4d.SPLINETYPE_LINEAR)
    pave_right_spline.SetPoint(0, curb_right_spline.GetPoint(2))
    pave_right_spline.SetPoint(1, pave_right_spline.GetPoint(0) + c4d.Vector(pave_width, 0, 0))
    pave_right_spline.SetPoint(2, pave_right_spline.GetPoint(1) - c4d.Vector(0, pave_height + 0.1, 0))

    # Nurbs
    street_nurb = c4d.BaseObject(c4d.Osweep)
    street_nurb.SetPhong(True, True, c4d.utils.DegToRad(60.0))
    street_nurb[c4d.PHONGTAG_PHONG_USEEDGES] = True
    street_Tag = street_nurb.MakeTag(c4d.Ttexture)
    street_Tag[c4d.TEXTURETAG_MATERIAL] = street_mat
    street_Tag[c4d.TEXTURETAG_PROJECTION] = 6
    street_Tag[c4d.TEXTURETAG_OFFSETY] = street_tile[0] / 100
    street_Tag[c4d.TEXTURETAG_LENGTHY] = street_tile[1] / 100
    street_Tag[c4d.TEXTURETAG_OFFSETX] = street_tile[2] / 100
    street_Tag[c4d.TEXTURETAG_LENGTHX] = street_tile[3] / 100

    curb_nurb_left = c4d.BaseObject(c4d.Osweep)
    curb_nurb_left.SetPhong(True, True, c4d.utils.DegToRad(60.0))
    curb_nurb_left[c4d.PHONGTAG_PHONG_USEEDGES] = True
    curb_left_Tag = curb_nurb_left.MakeTag(c4d.Ttexture)
    curb_left_Tag[c4d.TEXTURETAG_MATERIAL] = curb_mat
    curb_left_Tag[c4d.TEXTURETAG_PROJECTION] = 6
    curb_left_Tag[c4d.TEXTURETAG_OFFSETY] = curb_tile[0] / 100
    curb_left_Tag[c4d.TEXTURETAG_LENGTHY] = curb_tile[1] / 100
    curb_left_Tag[c4d.TEXTURETAG_OFFSETX] = curb_tile[2] / 100
    curb_left_Tag[c4d.TEXTURETAG_LENGTHX] = curb_tile[3] / 100

    pave_nurb_left = c4d.BaseObject(c4d.Osweep)
    pave_nurb_left.SetPhong(True, True, c4d.utils.DegToRad(60.0))
    pave_nurb_left[c4d.PHONGTAG_PHONG_USEEDGES] = True
    pave_left_Tag = pave_nurb_left.MakeTag(c4d.Ttexture)
    pave_left_Tag[c4d.TEXTURETAG_MATERIAL] = pave_mat
    pave_left_Tag[c4d.TEXTURETAG_PROJECTION] = 6
    pave_left_Tag[c4d.TEXTURETAG_OFFSETY] = pave_tile[0] / 100
    pave_left_Tag[c4d.TEXTURETAG_LENGTHY] = pave_tile[1] / 100
    pave_left_Tag[c4d.TEXTURETAG_OFFSETX] = pave_tile[2] / 100
    pave_left_Tag[c4d.TEXTURETAG_LENGTHX] = pave_tile[3] / 100

    curb_nurb_right = c4d.BaseObject(c4d.Osweep)
    curb_nurb_right.SetPhong(True, True, c4d.utils.DegToRad(60.0))
    curb_nurb_right[c4d.PHONGTAG_PHONG_USEEDGES] = True
    curb_right_Tag = curb_nurb_right.MakeTag(c4d.Ttexture)
    curb_right_Tag[c4d.TEXTURETAG_MATERIAL] = curb_mat
    curb_right_Tag[c4d.TEXTURETAG_PROJECTION] = 6
    curb_right_Tag[c4d.TEXTURETAG_OFFSETY] = curb_tile[0] / 100
    curb_right_Tag[c4d.TEXTURETAG_LENGTHY] = curb_tile[1] / 100
    curb_right_Tag[c4d.TEXTURETAG_OFFSETX] = curb_tile[2] / 100
    curb_right_Tag[c4d.TEXTURETAG_LENGTHX] = curb_tile[3] / 100

    pave_nurb_right = c4d.BaseObject(c4d.Osweep)
    pave_nurb_right.SetPhong(True, True, c4d.utils.DegToRad(60.0))
    pave_nurb_right[c4d.PHONGTAG_PHONG_USEEDGES] = True
    pave_right_Tag = pave_nurb_right.MakeTag(c4d.Ttexture)
    pave_right_Tag[c4d.TEXTURETAG_MATERIAL] = pave_mat
    pave_right_Tag[c4d.TEXTURETAG_PROJECTION] = 6
    pave_right_Tag[c4d.TEXTURETAG_OFFSETY] = pave_tile[0] / 100
    pave_right_Tag[c4d.TEXTURETAG_LENGTHY] = pave_tile[1] / 100
    pave_right_Tag[c4d.TEXTURETAG_OFFSETX] = pave_tile[2] / 100
    pave_right_Tag[c4d.TEXTURETAG_LENGTHX] = pave_tile[3] / 100

    # Paths
    path_curb_left = path.GetClone()
    path_pave_left = path.GetClone()
    path_curb_right = path.GetClone()
    path_pave_right = path.GetClone()

    # Inserting Objects
    path.GetClone().InsertUnder(street_nurb)
    spline.InsertUnder(street_nurb)
    street_nurb.InsertUnder(null)

    path_curb_left.InsertUnder(curb_nurb_left)
    curb_left_spline.InsertUnder(curb_nurb_left)
    curb_nurb_left.InsertUnder(null)

    path_pave_left.InsertUnder(pave_nurb_left)
    pave_left_spline.InsertUnder(pave_nurb_left)
    pave_nurb_left.InsertUnder(null)

    path_curb_right.InsertUnder(curb_nurb_right)
    curb_right_spline.InsertUnder(curb_nurb_right)
    curb_nurb_right.InsertUnder(null)

    path_pave_right.InsertUnder(pave_nurb_right)
    pave_right_spline.InsertUnder(pave_nurb_right)
    pave_nurb_right.InsertUnder(null)

    street_nurb[c4d.SWEEPOBJECT_ROTATIONFROM] = rot
    curb_nurb_left[c4d.SWEEPOBJECT_ROTATIONFROM] = rot
    curb_nurb_right[c4d.SWEEPOBJECT_ROTATIONFROM] = rot
    pave_nurb_right[c4d.SWEEPOBJECT_ROTATIONFROM] = rot
    pave_nurb_left[c4d.SWEEPOBJECT_ROTATIONFROM] = rot

    if bevel == True:
        bevelD = c4d.BaseObject(431000028)
        bevelD[c4d.O_BEVEL_SELECTION_PHONG_BREAK] = True
        bevelD[c4d.O_BEVEL_MODE_SELECTION_ANGLE_VAL] = c4d.utils.DegToRad(40)
        bevelD[c4d.O_BEVEL_LIMIT] = True
        bevelD[c4d.O_BEVEL_RADIUS] = offset
        bevelD[c4d.O_BEVEL_SUB] = subd
        bevelD.InsertUnder(null)

    # connect=c4d.BaseObject(c4d.Oconnector)
    # connect[c4d.CONNECTOBJECT_WELD]=True
    # connect[c4d.CONNECTOBJECT_TOLERANCE]=0.1
    # connect[c4d.CONNECTOBJECT_PHONG_MODE]=0
    # connect[c4d.CONNECTOBJECT_TEXTURES]=True

    # null.InsertUnder(connect)
    return null


class Lamp():

    def __init__(self, obj1, path, count, pos, rot):
        if obj1 == None:
            self.obj = None
        else:
            self.obj = c4d.BaseObject(c4d.Oinstance)
            self.obj[c4d.INSTANCEOBJECT_LINK] = obj1

        self.cloner = c4d.BaseObject(1018544)
        self.tag = None
        self.path = path
        self.count = count
        self.pos = pos
        self.offset = pos[2]
        self.rot = rot
        # self.cloner=c4d.BaseObject(1018544)
        # self.cloner[c4d.ID_MG_MOTIONGENERATOR_MODE]=0
        # self.cloner[c4d.MG_OBJECT_LINK]=path
        # self.cloner[c4d.MG_SPLINE_MODE]=2
        # self.cloner[c4d.MG_SPLINE_COUNT]=count
        # self.cloner[c4d.MGCLONER_VOLUMEINSTANCES_MODE]=0
        # self.cloner[c4d.MG_SPLINE_SMOOTH]=False
        # self.cloner[c4d.ID_MG_TRANSFORM_POSITION]=pos
        # self.cloner[c4d.ID_MG_TRANSFORM_ROTATE]=rot
        # column.InsertUnder(self.cloner)
        # return self.cloner
        # self.effector=c4d.BaseObject(1018774)
        # self.effector[c4d.MGSPLINEEFFECTOR_SPLINE]=path

    def create(self):

        if self.obj == None:
            self.cloner = None
        else:
            # self.obj=c4d.BaseObject(c4d.Oinstance)
            # op[c4d.ID_USERDATA,37]

            # inex=c4d.InExcludeData()
            # inex.InsertObject(self.effector,1)
            self.obj[c4d.INSTANCEOBJECT_RENDERINSTANCE_MODE] = 1

            self.tag = self.cloner.MakeTag(c4d.Tsds)
            self.tag[c4d.SDSTAG_USE_SDS_STEPS] = True
            self.tag[c4d.SDSTAG_SUBEDITOR] = 0
            self.tag[c4d.SDSTAG_SUBRAY] = 0

            self.cloner[c4d.ID_MG_MOTIONGENERATOR_MODE] = 0
            self.cloner[c4d.MG_OBJECT_LINK] = self.path
            self.cloner[c4d.MG_OBJECT_ALIGN] = True
            self.cloner[c4d.MG_SPLINE_MODE] = 2
            self.cloner[c4d.MG_SPLINE_COUNT] = self.count
            self.cloner[c4d.MGCLONER_VOLUMEINSTANCES_MODE] = 0
            self.cloner[c4d.MG_SPLINE_SMOOTH] = False
            self.cloner[c4d.ID_MG_TRANSFORM_POSITION] = c4d.Vector(-self.pos[0], self.pos[1], self.pos[2])
            self.cloner[c4d.ID_MG_TRANSFORM_ROTATE] = self.rot
            self.cloner[c4d.MG_SPLINE_LOOP] = True
            self.cloner[c4d.MG_SPLINE_OFFSET] = self.offset / 100
            # self.cloner[c4d.ID_MG_MOTIONGENERATOR_EFFECTORLIST]=inex
            self.obj.InsertUnder(self.cloner)
            return self.cloner

def Move(railer, mirror, pos, size):
    if mirror == 1:
        counterr = 0
        listep = []
        #
        #
        for i in railer.GetAllPoints().copy():
            listep.append(c4d.Vector(i[0] * -1, i[1], i[2]))
        #
        railer.SetAllPoints(listep)

    try:
        counter = 0
        points_list = railer.GetAllPoints().copy()
        for i in points_list:
            points_list[counter] -= pos
            counter += 1
        railer.SetAllPoints(points_list)

        # rail_spline.SetRelRot(rail_spline.GetRelRot()+rail_rot)


    except:

        return

    railer[c4d.ID_BASEOBJECT_REL_SCALE] += size - 1
    return railer

# ==================================


class Sweep():

    def __init__(self, tile, mat):
        self.sweep = c4d.BaseObject(c4d.Osweep)
        self.tile = tile
        self.mat = mat
        self.sweep_rail_Tag = self.sweep.MakeTag(c4d.Ttexture)
        self.sweep.SetPhong(True, True, c4d.utils.DegToRad(30.0))
        self.sweep_rail_Tag[c4d.TEXTURETAG_MATERIAL] = self.mat
        self.sweep_rail_Tag[c4d.TEXTURETAG_PROJECTION] = 6
        self.sweep_rail_Tag[c4d.TEXTURETAG_OFFSETY] = self.tile[0] / 100
        self.sweep_rail_Tag[c4d.TEXTURETAG_LENGTHY] = self.tile[1] / 100
        self.sweep_rail_Tag[c4d.TEXTURETAG_OFFSETX] = self.tile[2] / 100
        self.sweep_rail_Tag[c4d.TEXTURETAG_LENGTHX] = self.tile[3] / 100


class LoadProfile():
    def __init__(self):
        self.spline2 = None

    def create(self, num):
        # old
        # global_path = os.path.dirname(
        #     c4d.storage.GeGetC4DPath(c4d.C4D_PATH_PREFS)) + "\library\scripts\Street-Builder R23"

        global_path = os.path.join(c4d.storage.GeGetC4DPath(c4d.C4D_PATH_LIBRARY_USER), "scripts", "Street-Builder R23")

        path1 = os.path.join(global_path, "pro_1.file")
        path2 = os.path.join(global_path, "pro_2.file")
        path3 = os.path.join(global_path, "pro_3.file")
        path4 = os.path.join(global_path, "pro_4.file")
        path5 = os.path.join(global_path, "pro_5.file")
        path6 = os.path.join(global_path, "pro_6.file")

        spline = c4d.SplineObject(0, 0)

        if num == 0:
            c4d.storage.ReadHyperFile(doc, spline, path1, 100)

            self.spline2 = c4d.SplineObject(0, 0)
            if spline.GetSegmentCount != 0:
                self.spline2.ResizeObject(spline.GetPointCount(), spline.GetSegmentCount())
                for i in range(self.spline2.GetSegmentCount()):
                    self.spline2.SetSegment(i, spline.GetSegment(i)['cnt'], True)

                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

            else:
                self.spline2.ResizeObject(spline.GetPointCount(), 0)
                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

        elif num == 1:

            c4d.storage.ReadHyperFile(doc, spline, path2, 100)
            #
            self.spline2 = c4d.SplineObject(0, 0)
            if spline.GetSegmentCount != 0:
                self.spline2.ResizeObject(spline.GetPointCount(), spline.GetSegmentCount())
                for i in range(self.spline2.GetSegmentCount()):
                    self.spline2.SetSegment(i, spline.GetSegment(i)['cnt'], True)

                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

            else:
                self.spline2.ResizeObject(spline.GetPointCount(), 0)
                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

        elif num == 2:

            c4d.storage.ReadHyperFile(doc, spline, path3, 100)
            #
            self.spline2 = c4d.SplineObject(0, 0)
            if spline.GetSegmentCount != 0:
                self.spline2.ResizeObject(spline.GetPointCount(), spline.GetSegmentCount())
                for i in range(self.spline2.GetSegmentCount()):
                    self.spline2.SetSegment(i, spline.GetSegment(i)['cnt'], True)

                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

            else:
                self.spline2.ResizeObject(spline.GetPointCount(), 0)
                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

        elif num == 3:

            c4d.storage.ReadHyperFile(doc, spline, path4, 100)
            #
            self.spline2 = c4d.SplineObject(0, 0)
            if spline.GetSegmentCount != 0:
                self.spline2.ResizeObject(spline.GetPointCount(), spline.GetSegmentCount())
                for i in range(self.spline2.GetSegmentCount()):
                    self.spline2.SetSegment(i, spline.GetSegment(i)['cnt'], True)

                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

            else:
                self.spline2.ResizeObject(spline.GetPointCount(), 0)
                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True
        elif num == 4:

            c4d.storage.ReadHyperFile(doc, spline, path5, 100)
            #
            self.spline2 = c4d.SplineObject(0, 0)
            if spline.GetSegmentCount != 0:
                self.spline2.ResizeObject(spline.GetPointCount(), spline.GetSegmentCount())
                for i in range(self.spline2.GetSegmentCount()):
                    self.spline2.SetSegment(i, spline.GetSegment(i)['cnt'], True)

                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

            else:
                spline2.ResizeObject(spline.GetPointCount(), 0)
                spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                spline2[c4d.SPLINEOBJECT_CLOSED] = True

        elif num == 5:

            c4d.storage.ReadHyperFile(doc, spline, path6, 100)
            #
            self.spline2 = c4d.SplineObject(0, 0)
            if spline.GetSegmentCount != 0:
                self.spline2.ResizeObject(spline.GetPointCount(), spline.GetSegmentCount())
                for i in range(self.spline2.GetSegmentCount()):
                    self.spline2.SetSegment(i, spline.GetSegment(i)['cnt'], True)

                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

            else:
                self.spline2.ResizeObject(spline.GetPointCount(), 0)
                self.spline2.SetAllPoints(spline.GetAllPoints())
                # spline2[c4d.ID_BASEOBJECT_REL_SCALE]+=size-1
                self.spline2[c4d.SPLINEOBJECT_CLOSED] = True

        else:
            self.spline2 = None

        return self.spline2


def main():
    comp_list = [c4d.Ospline,
                c4d.Ospline4side,
                c4d.Osplinearc,
                c4d.Osplinecircle,
                c4d.Osplinecissoid,
                c4d.Osplinecogwheel,
                c4d.Osplinecontour,
                c4d.Osplinecycloid,
                c4d.Osplinedeformer,
                c4d.Osplineflower,
                c4d.Osplineformula,
                c4d.Osplinehelix,
                c4d.Osplinenside,
                c4d.Osplineprimitive,
                c4d.Osplineprofile,
                c4d.Osplinerail,
                c4d.Osplinerectangle,
                c4d.Osplinestar,
                c4d.Osplinetext]
                
    comp_spline = op.GetDown()
    if comp_spline and comp_spline.GetType() in comp_list:
        comp_spline = comp_spline.GetType()
                
    if len(op.GetChildren()) == 1 and op.GetDown().IsInstanceOf(comp_spline):  # bool(op.GetDown().GetInfo() & c4d.OBJECT_ISSPLINE):

        global path
        global pave_width
        global curb_width
        global street_width
        global pave_height

        global rot
        global bevel
        global offset
        global subd

        global pave_mat
        global pave_tile
        global curb_mat
        global curb_tile
        global street_mat
        global street_tile

        # global obj1
        global obj1_pos
        global obj1_rot
        global obj1_count
        # global obj2
        global obj2_pos
        global obj2_rot
        global obj2_count

        global rail_bool
        global rail_type
        global rail
        global rail_pos
        # global rail_rot
        global rail_mirror
        global rail_size
        global rail_mat
        global rail_tile
        # global column
        global column_pos
        global column_rot
        global column_count
        global rail_size

        global rail2_bool
        global rail2_type
        global rail2
        global rail2_pos
        # global rail_rot
        global rail2_mirror
        global rail2_size
        global rail2_mat
        global rail2_tile
        global column2
        global column2_pos
        global column2_rot
        global column2_count

        global rail3_bool
        global rail3_type
        global rail3
        global rail3_pos
        # global rail_rot
        global rail3_mirror
        global rail3_size
        global rail3_mat
        global rail3_tile
        global column3
        global column3_pos
        global column3_rot
        global column3_count

        pave_width = op[c4d.ID_USERDATA, 3]
        curb_width = op[c4d.ID_USERDATA, 4]
        street_width = op[c4d.ID_USERDATA, 5]
        pave_height = op[c4d.ID_USERDATA, 6]
        rot = op[c4d.ID_USERDATA, 7]
        bevel = op[c4d.ID_USERDATA, 8]
        offset = op[c4d.ID_USERDATA, 9]
        subd = op[c4d.ID_USERDATA, 10]
        pave_mat = op[c4d.ID_USERDATA, 12]
        pave_tile = op[c4d.ID_USERDATA, 13]

        pave_tile = c4d.Vector4d()
        pave_tile[0] = op[c4d.ID_USERDATA, 13][0]
        pave_tile[1] = op[c4d.ID_USERDATA, 13][1]
        pave_tile[2] = op[c4d.ID_USERDATA, 13][2]
        pave_tile[3] = op[c4d.ID_USERDATA, 13][3]

        curb_mat = op[c4d.ID_USERDATA, 14]
        curb_tile = c4d.Vector4d()
        curb_tile[0] = op[c4d.ID_USERDATA, 15][0]
        curb_tile[1] = op[c4d.ID_USERDATA, 15][1]
        curb_tile[2] = op[c4d.ID_USERDATA, 15][2]
        curb_tile[3] = op[c4d.ID_USERDATA, 15][3]

        street_mat = op[c4d.ID_USERDATA, 16]
        street_tile = c4d.Vector4d()
        street_tile[0] = op[c4d.ID_USERDATA, 17][0]
        street_tile[1] = op[c4d.ID_USERDATA, 17][1]
        street_tile[2] = op[c4d.ID_USERDATA, 17][2]
        street_tile[3] = op[c4d.ID_USERDATA, 17][3]

        obj1 = op[c4d.ID_USERDATA, 19]
        obj1_pos = op[c4d.ID_USERDATA, 20]
        obj1_rot = op[c4d.ID_USERDATA, 21]
        obj1_count = op[c4d.ID_USERDATA, 22]
        obj2 = op[c4d.ID_USERDATA, 23]
        obj2_pos = op[c4d.ID_USERDATA, 24]
        obj2_rot = op[c4d.ID_USERDATA, 25]
        obj2_count = op[c4d.ID_USERDATA, 26]

        rail_bool = op[c4d.ID_USERDATA, 29]
        rail_type = [c4d.ID_USERDATA, 30]
        rail = op[c4d.ID_USERDATA, 31]

        rail_pos = op[c4d.ID_USERDATA, 32]
        # rail_rot=op[c4d.ID_USERDATA,33]
        rail_size = op[c4d.ID_USERDATA, 33]
        rail_mirror = op[c4d.ID_USERDATA, 34]

        rail_mat = op[c4d.ID_USERDATA, 35]
        rail_tile = c4d.Vector4d()
        rail_tile[0] = op[c4d.ID_USERDATA, 36][0]
        rail_tile[1] = op[c4d.ID_USERDATA, 36][1]
        rail_tile[2] = op[c4d.ID_USERDATA, 36][2]
        rail_tile[3] = op[c4d.ID_USERDATA, 36][3]

        column = op[c4d.ID_USERDATA, 37]
        column_pos = op[c4d.ID_USERDATA, 38]
        column_rot = op[c4d.ID_USERDATA, 39]
        column_count = op[c4d.ID_USERDATA, 40]

        # Rail2=============
        rail2_bool = op[c4d.ID_USERDATA, 42]
        rail2_type = [c4d.ID_USERDATA, 43]
        rail2 = op[c4d.ID_USERDATA, 44]
        rail2_pos = op[c4d.ID_USERDATA, 45]
        # rail_rot=op[c4d.ID_USERDATA,33]
        rail2_size = op[c4d.ID_USERDATA, 46]
        rail2_mirror = op[c4d.ID_USERDATA, 47]

        rail2_mat = op[c4d.ID_USERDATA, 48]
        rail2_tile = c4d.Vector4d()
        rail2_tile[0] = op[c4d.ID_USERDATA, 49][0]
        rail2_tile[1] = op[c4d.ID_USERDATA, 49][1]
        rail2_tile[2] = op[c4d.ID_USERDATA, 49][2]
        rail2_tile[3] = op[c4d.ID_USERDATA, 49][3]

        column2 = op[c4d.ID_USERDATA, 50]
        column2_pos = op[c4d.ID_USERDATA, 51]
        column2_rot = op[c4d.ID_USERDATA, 52]
        column2_count = op[c4d.ID_USERDATA, 53]
        # =========================

        # Rail3=============
        rail3_bool = op[c4d.ID_USERDATA, 55]
        rail3_type = [c4d.ID_USERDATA, 56]
        rail3 = op[c4d.ID_USERDATA, 57]
        rail3_pos = op[c4d.ID_USERDATA, 58]
        # rail_rot=op[c4d.ID_USERDATA,33]
        rail3_size = op[c4d.ID_USERDATA, 59]
        rail3_mirror = op[c4d.ID_USERDATA, 60]

        rail3_mat = op[c4d.ID_USERDATA, 61]
        rail3_tile = c4d.Vector4d()
        rail3_tile[0] = op[c4d.ID_USERDATA, 62][0]
        rail3_tile[1] = op[c4d.ID_USERDATA, 62][1]
        rail3_tile[2] = op[c4d.ID_USERDATA, 62][2]
        rail3_tile[3] = op[c4d.ID_USERDATA, 62][3]

        column3 = op[c4d.ID_USERDATA, 63]
        column3_pos = op[c4d.ID_USERDATA, 64]
        column3_rot = op[c4d.ID_USERDATA, 65]
        column3_count = op[c4d.ID_USERDATA, 66]
        # =========================

        # ====Pitch=============
        global look_up
        global pitch
        look_up = op[c4d.ID_USERDATA, 67]
        pitch = op[c4d.ID_USERDATA, 68]

        # ==================

        path = op.GetDown()
        container = c4d.BaseObject(c4d.Onull)
        street = Spline()

        # Effector==========
        class Effector():
            def __init__(self, rot):
                self.effector = c4d.BaseObject(1018889)
                self.effector[c4d.ID_BASEOBJECT_GENERATOR_FLAG] = look_up
                self.effector[c4d.MGTARGETEFFECTOR_PITCH] = pitch
                self.effector[c4d.MGTARGETEFFECTOR_REVERSEVECTOR] = True
                self.simple = c4d.BaseObject(1021337)
                self.simple[c4d.ID_MG_BASEEFFECTOR_ROTATE_ACTIVE] = True
                self.simple[c4d.ID_MG_BASEEFFECTOR_POSITION_ACTIVE] = False
                self.simple[c4d.ID_MG_BASEEFFECTOR_ROTATION] = rot

                self.effectData = c4d.InExcludeData()
                self.effectData.InsertObject(self.effector, 1)
                self.effectData.InsertObject(self.simple, 1)

        # ==================

        # Lamp1=======================
        if obj1 != None:

            lamp1 = Lamp(obj1, path, obj1_count, obj1_pos, obj1_rot)
            lamp1.create()
            if look_up == True:
                obj1_effector = Effector(obj1_rot)

                lamp1.cloner[c4d.ID_MG_MOTIONGENERATOR_EFFECTORLIST] = obj1_effector.effectData
                obj1_effector.effector.InsertUnder(container)
                obj1_effector.simple.InsertUnder(container)
            lamp1.cloner.InsertUnder(container)
        # ===================================================

        # Lamp2=======================
        if obj2 != None:

            lamp2 = Lamp(obj2, path, obj2_count, obj2_pos, obj2_rot)
            lamp2.create()
            if look_up == True:
                obj2_effector = Effector(obj2_rot)

                lamp2.cloner[c4d.ID_MG_MOTIONGENERATOR_EFFECTORLIST] = obj2_effector.effectData
                obj2_effector.effector.InsertUnder(container)
                obj2_effector.simple.InsertUnder(container)

            lamp2.cloner.InsertUnder(container)
        # ===================================================

        # Rail 1======================
        if rail_bool == True:
            profile = LoadProfile()
            profile.create(op[c4d.ID_USERDATA, 30])

            if rail == None:  # or bool(rail.GetInfo() & c4d.OBJECT_GENERATOR):
                # patho=path.GetClone()
                sweep = Sweep(rail_tile, rail_mat)

                path.GetClone().InsertUnder(sweep.sweep)
                profile.spline2.InsertUnder(sweep.sweep)
                Move(profile.spline2, rail_mirror, rail_pos, rail_size)
                sweep.sweep.InsertUnder(container)

            elif rail.IsInstanceOf(c4d.Ospline):
                sweep = Sweep(rail_tile, rail_mat)

                path.GetClone().InsertUnder(sweep.sweep)
                profile = rail.GetClone()
                profile.InsertUnder(sweep.sweep)
                Move(profile, rail_mirror, rail_pos, rail_size)
                sweep.sweep.InsertUnder(container)
            else:
                sweep = Sweep(rail_tile, rail_mat)

                path.GetClone().InsertUnder(sweep.sweep)
                profile.spline2.InsertUnder(sweep.sweep)
                Move(profile.spline2, rail_mirror, rail_pos, rail_size)
                sweep.sweep.InsertUnder(container)

        # Column1===================================
        if column:
            rail_column = Lamp(column, path, column_count, column_pos, column_rot)
            rail_column.create()
            if look_up == True:
                rail_column_effector = Effector(column_rot)

                rail_column.cloner[c4d.ID_MG_MOTIONGENERATOR_EFFECTORLIST] = rail_column_effector.effectData
                rail_column_effector.effector.InsertUnder(container)
                rail_column_effector.simple.InsertUnder(container)
            rail_column.cloner.InsertUnder(container)

        # ============================================================

        # Rail 2======================
        if rail2_bool == True:
            profile2 = LoadProfile()
            profile2.create(op[c4d.ID_USERDATA, 43])

            if rail2 == None:  # or bool(rail.GetInfo() & c4d.OBJECT_GENERATOR):
                # path2=path.GetClone()
                sweep2 = Sweep(rail2_tile, rail2_mat)
                #
                #
                path.GetClone().InsertUnder(sweep2.sweep)
                profile2.spline2.InsertUnder(sweep2.sweep)

                Move(profile2.spline2, rail2_mirror, rail2_pos, rail2_size)

                sweep2.sweep.InsertUnder(container)

            #
            elif rail2.IsInstanceOf(c4d.Ospline):
                sweep2 = Sweep(rail2_tile, rail2_mat)
                #
                path.GetClone().InsertUnder(sweep2.sweep)
                profile2 = rail2.GetClone()
                profile2.InsertUnder(sweep2.sweep)
                Move(profile2, rail2_mirror, rail2_pos, rail2_size)
                sweep2.sweep.InsertUnder(container)
            else:
                sweep2 = Sweep(rail2_tile, rail2_mat)
                #
                path.GetClone().InsertUnder(sweep2.sweep)
                profile2.spline2.InsertUnder(sweep2.sweep)
                Move(profile2.spline2, rail2_mirror, rail2_pos, rail2_size)
                sweep2.sweep.InsertUnder(container)

        # Column2===================================
        if column2:
            rail2_column = Lamp(column2, path, column2_count, column2_pos, column2_rot)
            rail2_column.create()
            if look_up == True:
                rail2_column_effector = Effector(column2_rot)

                rail2_column.cloner[c4d.ID_MG_MOTIONGENERATOR_EFFECTORLIST] = rail2_column_effector.effectData
                rail2_column_effector.effector.InsertUnder(container)
                rail2_column_effector.simple.InsertUnder(container)
            rail2_column.cloner.InsertUnder(container)

        # ============================================================

        # Rail 3======================
        if rail3_bool == True:
            profile3 = LoadProfile()
            profile3.create(op[c4d.ID_USERDATA, 56])

            if rail3 == None:  # or bool(rail.GetInfo() & c4d.OBJECT_GENERATOR):
                # path2=path.GetClone()
                sweep3 = Sweep(rail3_tile, rail3_mat)

                path.GetClone().InsertUnder(sweep3.sweep)
                profile3.spline2.InsertUnder(sweep3.sweep)

                Move(profile3.spline2, rail3_mirror, rail3_pos, rail3_size)

                sweep3.sweep.InsertUnder(container)

            elif rail3.IsInstanceOf(c4d.Ospline):
                sweep3 = Sweep(rail3_tile, rail3_mat)
                path.GetClone().InsertUnder(sweep3.sweep)
                profile3 = rail3.GetClone()
                profile3.InsertUnder(sweep3.sweep)
                Move(profile3, rail3_mirror, rail3_pos, rail3_size)
                sweep3.sweep.InsertUnder(container)
            else:
                sweep3 = Sweep(rail3_tile, rail3_mat)
                path.GetClone().InsertUnder(sweep3.sweep)
                profile3.spline2.InsertUnder(sweep3.sweep)
                Move(profile3.spline2, rail3_mirror, rail3_pos, rail3_size)
                sweep3.sweep.InsertUnder(container)

        # Column2===================================
        if column3:
            rail3_column = Lamp(column3, path, column3_count, column3_pos, column3_rot)
            rail3_column.create()
            if look_up == True:
                rail3_column_effector = Effector(column3_rot)

                rail3_column.cloner[c4d.ID_MG_MOTIONGENERATOR_EFFECTORLIST] = rail3_column_effector.effectData
                rail3_column_effector.effector.InsertUnder(container)
                rail3_column_effector.simple.InsertUnder(container)
            rail3_column.cloner.InsertUnder(container)

        # ============================================================

        street.InsertUnder(container)
        return container

    else:
        return



    '''

    doc.InsertObject(python)
    c4d.EventAdd()


# Execute main()
if __name__ == '__main__':
    FOLDER_PATH = os.path.split(__file__)[0]
    main()